'use strict';

import fs from 'fs';
import rfr from 'rfr';
import path from 'path';
import fsx from 'fs-extra';
import {default as createDebugger} from 'debug';

const debug = createDebugger('quture:config');

/**
 *
 */
export default class Config {

  /**
   * Create a new Config instance.
   *
   * @param {object} options
   * @returns {Config}
   * @throws {Error} if a configuration option is not a valid option
   */
  constructor(options) {
    if (options.configDir && typeof options.configDir === 'string') {
      this._configDir = options.configDir;

      if (!options.dirSync || options.dirSync === true) {
        Config.ensureDirSync(this._configDir);
      }

      this._rfr = rfr({ root: this._configDir });
    } else {
      throw new Error('{options.configDir} is required');
    }
  }

  /**
   * Ensure the config is created.
   *
   * @param {string} dirPath
   */
  static ensureDirSync(dirPath) {
    if (!fs.existsSync(dirPath)) {
      fsx.ensureDirSync(dirPath);
    } else {
      if (!fs.lstatSync(dirPath).isDirectory()) {
        throw new Error('Unable to create a directory at [' + dirPath + '] because a file with the name [' + path.parse(dirPath).base + '] already exists');
      }
    }
  }

  /**
   * Create the configuration directory if it doesn't exist.
   */
  createDirIfNotExists() {
    Config.ensureDirSync(this._configDir);
  }

  /**
   * Retrieve the path to the config directory.
   *
   * @returns {string}
   */
  getConfigDirectory() {
    return this._configDir;
  }

  /**
   * Get a value from an object specified within a file.
   *
   * @param {string} fileName
   * @param {string} defaultValue
   * @returns value
   */
  get(fileName, defaultValue) {
    let fileNameSplit = fileName.split('.');
    let value = null;
    try {
      debug('Attempting to load a configuration value from ' + fileNameSplit[0]);
      let file = this._rfr(fileNameSplit[0]);
      value = file;
      for (let i = 1; i < fileNameSplit.length; i++) {
        let exists = false;
        if (i === 1) {
          if (fileNameSplit[i] != null) {
            value = file[fileNameSplit[i]];
            exists = true;
          }
        } else {
          if (value != null) {
            if (fileNameSplit[i] != null) {
              value = value[fileNameSplit[i]];
              exists = true;

            }
          }
        }

        if (exists) {
          debug('Configuration value loaded from \'' + fileNameSplit[0] + '\'');
          break;
        }

        if (!exists) {
          debug('\'' + fileNameSplit[0] + '\' doesn\'t exist within \'' + fileName);
          break;
        }
      }
    } catch(e) {
      debug(e);
      debug('Unable to load file with the name \'' + fileNameSplit[0] + '\'');
    }

    if (value && typeof value === 'string') {
      value = (value.toLowerCase() === 'true' || value.toLowerCase() === 'false') ? (value.toLowerCase() === 'true' ? true : false) : value;
    }

    return (value !== undefined && value !== null) ? value : defaultValue;
  }

  /**
   * Check if the specified path exists.
   *
   * @param {string} configPath
   * @returns {boolean}
   */
  has(configPath) {
    return (this.get(configPath) !== null && this.get(configPath) !== undefined);
  }

}
